'
' SpecialFolders (VBScript).vbs
'
'	Altap Salamander Automation Script.
'
' Calls SpecialFolders.hta to presents a list of special folders in Windows.
' The user can select one and execute available verbs on it e.g. to open it as
' a folder in Windows Explorer. For special folders that have a representation 
' in the file system, the path can be opened in the active panel in Salamander.
'
' This software is in the public domain.
'
' written by Stefan von Allmen
' 
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
' EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
' OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
' NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
' HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
' OTHER DEALINGS IN THE SOFTWARE.
'
' =======================================================
'
' Change History
' 2012-06-14	v0.1	Initial version
' 2012-06-15	v0.2	mru implemented
'			Return CSIDL No. instead of path
'			Return through Environment instead of registry
' 2012-06-18	v0.3	getReturn allways returned -1
' 2012-06-21	v0.4	Checked code with Option Explicit
'			Internationalisation
'			Option dialog: history depth (enabled with Parameter /opt)
' 2012-06-22	v1.0	First release

Option Explicit
' cannot detect host (Salamander or WScript) if option explicit is on

Dim MruMax, Finished, Param, mru, retval, i
Dim WshShell, objShell, objFolder, objFolderItem, dctMru

Const ScriptName = "Special Folders"
Const ScriptVersion = "v0.3"
Const PersistentValMru = "SpFldr.mru"
Const PersistentValHistoryDepth = "SpFldr.MruMax"
Const VolatileEnvCsidl = "SPFLDR_CSIDL" 

' i18n strings
Const lngHistoryDepth = "&History depth"
Const lngHistoryNone = "Set to 0 to disable history."
Const lngHistoryAll = "Set negative value to extend history to all,"
Const lngHistoryAll2 = "not showing option groups."
Const lngInvalidNumber = "The value is not numeric."
Const lngOk = "O&k"
Const lngCancel = "&Cancel"
Const lngOptions = "Options"
const lngWaitWindowText = "Please go to the ""Special Folders"" window and close it in order to continue with Altap Salamander."

Finished = False
Set WshShell = CreateObject("WScript.Shell")
Do Until Finished
Finished = True
'Run the hta.
If IsEmpty(Salamander) Then
    Param = "/env:" & VolatileEnvCsidl
    If WScript.Arguments.Count > 0 Then
        For Each Arg in WScript.Arguments.Count-1
            Arg = WScript.Arguments(i)
            If InStr(1, Arg, "/env:", 1) = 1 Then
                Arg = "/env:" & VolatileEnvCsidl
            End If 
            If i = 0 Then 
                Param = Arg
            Else
                Param = Param & " " & Arg
            End If 
        Next
    End If
'MsgBox Param, 0, "HTA Parameters"
    WshShell.Run """" & mid(WScript.ScriptFullName, 1, InStrRev(WScript.ScriptFullName, ".")) & "hta""" & Param, 1, True
Else
    Salamander.WaitWindow.CanCancel = False
    Salamander.WaitWindow.Title = ScriptName & " " & ScriptVersion
    Salamander.WaitWindow.Text = lngWaitWindowText
    Salamander.WaitWindow.Show
    MruMax = Salamander.GetPersistentVal(PersistentValHistoryDepth)
    If Not IsNumeric(MruMax) Then 
       If Not IsNull(MruMax) Then Salamander.SetPersistentVal PersistentValHistoryDepth, Null
       MruMax = 6
    End If

    Param = " /opt /env:" & VolatileEnvCsidl
    mru = Salamander.GetPersistentVal(PersistentValMru)
    If Not IsNull(mru) Then
        Param = Param & " /mru:" & mru
    End If
    If MruMax < 0 Then
        Param = Param & " /no-grp"
    End If
'MsgBox Param, 0, "HTA Parameters"
'If Salamander.MsgBox(mid(Salamander.Script.Path, 1, InStrRev(Salamander.Script.Path, ".")) & "hta""", 1, ScriptName) = 2 Then Salamander.AbortScript
    WshShell.Run """" & mid(Salamander.Script.Path, 1, InStrRev(Salamander.Script.Path, ".")) & "hta""" & Param, 1, True
End If

retval = getReturn
If retval <> -1 Then
  If RetVal = "OPTIONS" Then
    Finished = False
    SetOptions
  Else
    Set objShell = CreateObject("Shell.Application")
    Set objFolder = objShell.Namespace(Eval(retval))
    If Not (objFolder Is Nothing) Then
        Set objFolderItem = objFolder.Self
        If IsEmpty(Salamander) Then
            If IsEmpty(WScript.StdIn) Then
                MsgBox objFolderItem.Path, 0, ScriptName
            Else
                WScript.Echo objFolderItem.Path
            End If
        Else
            If MruMax <> 0 Then
                Set dctMru = CreateObject("Scripting.Dictionary")
                dctMru.Add retval, Null
                i = 1
                If Not IsNull(mru) Then
                    For Each retval in split(mru, ",")
                        If Not dctMru.Exists(retval) Then
                           dctMru.Add retval, Null
                           i = i + 1
                           If (i >= MruMax) And (MruMax > 0) Then Exit For
                        End If
                    Next
                End If
                Salamander.SetPersistentVal PersistentValMru, join(dctMru.Keys, ",")
            Else
                If Not IsNull(mru) Then
                    ' Delete an existing value
                    Salamander.SetPersistentVal PersistentValMru, Null
                End If
            End If
            Salamander.SourcePanel.Path = objFolderItem.Path
        End If
    End If
  End If
End If 
Loop
Set WshShell = Nothing

'Read the registry entry created by the hta and delete it.
Function getReturn
    Dim objEnvironment
    Set objEnvironment = WshShell.Environment("Volatile")
    If IsEmpty(objEnvironment.Item(VolatileEnvCsidl)) Or IsNull(objEnvironment.Item(VolatileEnvCsidl)) Then
        getReturn = -1
    Else
        getReturn = objEnvironment.Item(VolatileEnvCsidl)
        ' a non-existent Env variable does neither return Empty nor Null nor
        ' is there a Function to check for it, but removal yields an error
        ON ERROR RESUME NEXT
        objEnvironment.Remove VolatileEnvCsidl
        If Err.Number <> 0 Then
            ERR.Clear
            getReturn = -1
        End If
        ON ERROR GOTO 0
    End If
    Set objEnvironment = Nothing
End Function

Sub SetOptions
    Dim fm, Res, Val
    Do
        set fm = Salamander.Forms.Form(ScriptName & " - " & lngOptions)
        fm.lblMruMax = Salamander.Forms.Label(lngHistoryDepth)
        fm.inpMruMax = Salamander.Forms.Textbox(MruMax)
        fm.lbltext1 = Salamander.Forms.Label(lngHistoryNone)
        fm.lbltext2 = Salamander.Forms.Label(lngHistoryAll)
        fm.lbltext3 = Salamander.Forms.Label(lngHistoryAll2)
        fm.lbltext4 = Salamander.Forms.Label(" ") 'spacer
        fm.btnOk = Salamander.Forms.Button(lngOk, 1)
        fm.btnCancel = Salamander.Forms.Button(lngCancel, 2)
        Res = fm.execute
        If Res = 2 Then Exit Do
        If Res = 1 Then
            Val = fm.inpMruMax.Text
            If IsNumeric(Val) Then
                MruMax = CInt(Val)
                Salamander.SetPersistentVal PersistentValHistoryDepth, MruMax
                Exit Do
            End If
            If Salamander.MsgBox(lngInvalidNumber, 21, Replace(lngHistoryDepth, "&", "")) = 2 Then ' Retry / Cancel 
                'Cancel
                Exit Do
            End If
        End If 
    Loop
    Set fm = Nothing
End Sub